/**
 * Functions available for implementation:
 *
 * getColumns() - return columns for metalog
 * getIdleColors() - return colors for metalog of idle event
 * getIdleMetaLog() - return text for metalog of idle event
 * handleLine(num, line) - handle function that is called every line that is read by tail process
 *
 * Functions that are part of the API:
 *
 * log(<message>, <fgcolor>, <bgcolor> - log in colorview the <message>, returns the <line number> in colorview.
 * metalog(<line number>, [<column text>, ...], <fgcolor>, <bgcolor> - add meta log linked to <line number> in colorview. uses text given in column text-array.
 * md5(<string>) - calculate md5 hash for given <string>
 */

/////////// User variables /////////// 

var processing = false;
var buffer = [];
var config = {
	'SELECT'          : { 'keyword' : 'FROM',        'fgcolor' : 'black',   'bgcolor' : '#F0F0F0' },
	'(' /* UNION */   : { 'keyword' : 'FROM',        'fgcolor' : 'black',   'bgcolor' : '#F0F0F0' },
	'EXPLAIN'         : { 'keyword' : 'FROM',        'fgcolor' : 'black',   'bgcolor' : '#F0F0F0' },
	'INSERT'          : { 'keyword' : 'INTO',        'fgcolor' : '#008080', 'bgcolor' : '#F0F0F0' },
	'CREATE'          : { 'keyword' : 'INTO',        'fgcolor' : '#008080', 'bgcolor' : '#F0F0F0' },
	'REPLACE'         : { 'keyword' : 'INTO',        'fgcolor' : '#008080', 'bgcolor' : '#F0F0F0' },
	'UPDATE'          : { 'keyword' : 'UPDATE',      'fgcolor' : 'orange',  'bgcolor' : '#F0F0F0' },
	'DELETE'          : { 'keyword' : 'FROM',        'fgcolor' : 'red',     'bgcolor' : '#F0F0F0' },
	'DROP'            : { 'keyword' : 'FROM',        'fgcolor' : 'red',     'bgcolor' : '#F0F0F0' },
	'START'           : { 'keyword' : 'TRANSACTION', 'fgcolor' : 'blue',    'bgcolor' : '#F0F0F0' },
	'COMMIT'          : { 'keyword' : 'COMMIT',      'fgcolor' : 'blue',    'bgcolor' : '#F0F0F0' },
	'**fallback**'    : { 'keyword' : '',            'fgcolor' : '#c0c0c0', 'bgcolor' : '#F0F0F0' }
};

/////////// Implemented API functions /////////// 

function getColumns()
{
	return [['Origin', 80], ['Timestamp', 75], ['MTID', 70], ['Thread', 170], ['Combined', 600], ['Hash', 80]];
}

function getIdleColors()
{
	return ['white', 'black'];
}

// If you do not want the IDLE marker, simply remove the following impl.
function getIdleMetaLog()
{
	return ['IDLE', '---', '---', '---', '@@message@@', '---']; // @@message@@ is replaced with "No messages for xxx seconds.."
}

function handleLine(num, line)
{

	if (line.match(/^\[\d{2}-[a-zA-Z]{3}-\d{4} \d{2}:\d{2}:\d{2} UTC\] /))
	{
		// from php error_log
		var num = log(line);
		if (line.match(/error/))
			metalog(num, ['PHP', line.substr(13, 8), '', 'Exception', line.substr(27)], 'black', 'red');
		
		return;
	}


	// Start buffering lines
	if (line == '@@begin@@') {
		processing = true;
		return;
	}

	if (!processing)
		return;


	// Process buffered lines
	if (line == '@@end@@') {
		try {
			processBuffer();
		}
		catch (e) {
			metalog(log('Exception occured in processBuffer(): ' + e.message, 'black', 'red'),
					['exception'], 'black', 'red');
		}
		return;
	}

	// Buffer lines
	buffer.push(line);
}

/////////// User functions /////////// 

function processBuffer()
{
	// Get params
	if (buffer.length < 4)
		return;

        // log block header
	var date = buffer.shift(), time = buffer.shift(), thread_id = buffer.shift();
	log(date + ' ' + time + ' ' + thread_id, '#00FF80', '#0000ff');

	// log contents
	var sql = buffer, sqlmin = buffer.join(" ").trim();
	var firstline = 0;
	for (var i=0; i<sql.length; i++) {
		var num = log([sql[i].replace(/\t/g, '    '), sql[i]]);
		if (i == 0)
			firstline = num;
	}

	// construct some meta data
	var source = sqlmin.split(' ')[0].toUpperCase();
	var logdata = config[source] || config['**fallback**'];
	var metalogmsg = sqlmin;
	var operation = source == '(' ? '' : source;
	var table = '';
	var tokens = sqlmin.split(logdata.keyword);
	if (tokens.length >= 2) {
		tokens = tokens[1].trim().split(' ');
		if (tokens.length > 0)
			table = tokens[0];
	}

	// Add metalog
	metalog(firstline, 
		['MySQL', time, operation, table, metalogmsg, md5(metalogmsg)], 
		logdata.fgcolor, 
		logdata.bgcolor);

	// Empty buffer
	buffer = [];
}


/////////// Utility functions /////////// 

function str_pad_left(string, pad, length)
{
	return (new Array(length + 1).join(pad) + string).slice(-length);
}

function str_pad_right(string, pad, length)
{
	return (string + new Array(length+1).join(pad)).slice(0, length);
}

